// PPM tester for ATtiny85 linked to SSD1306 128x64 OLED display
// detects PPM signal (typically from a radio control transmitter buddy box port or similar) and displays up to 12 channels numerically and with an oscilloscope-style graphic
// by ceptimus 2016-10-14
//
// use the ATtiny85 capable of running at 20MHz (20PU).  Burn bootloader to operate on internal clock at 16 MHz
// the XTAL1 and XTAL2 pins of the chip are free if you want to fit a 16MHz crystal for greater timing accuracy (and burn bootloader to suit)
// but the chip's built-in oscillator seems accurate enough really for this application (less than 1% error in my tests)

// connect OLED display SCL to SCL (pin 7 of 8-pin DIL chip)
// connect OLED display SDA to SDA (pin 5 of 8-pin DIL chip)
// officially you're supposed to fit pull-up resistors (2K2 or similar) to SCL and SDA, but I found my example worked fine with no resistors
// connect 5V VCC supply to pin 8 of 8-pin DIL chip, GND to pin 5.
// no connections to pins 1, 2, 3 unless you're fitting a crystal between 2 and 3
#include "SSD1306_minimal.h"

// pin on which incoming PPM signal is applied.  Signal goes through 10k resistor then 0.1uF capacitor to this pin - this allows opposite polarity (negative going) pulses to also be sensed.
// this is pin 6 of the 8-pin DIL chip
#define PPM_IN 1

SSD1306_Mini oled;

uint32_t reading[12]; // PPM readings for up to 12 channels.  units are 1/16 microsecond (62.5 ps) to allow for smoothing back to microseconds
uint32_t pulse; // smoothed width of pulse in same 62.5 ps units.  Pulse width isn't really important as pulse position is measured between two leading edges.
uint32_t usStart, previous; // capture microsecond values for beginning of pulse train, and previous edge
uint8_t mode; // what stage of detection program is operating in
uint8_t channel; // keeps track of which channel of the PPM signal is being measured, and ultimately counts the number of channels (up to twelve)
bool highPulses; // flag that is set if PPM signal is mostly low with high going pulses.  flag is cleared for opposite polarity signal.

// wait till input is high then return.  Time out after 65535 loops (about 242 ms at 16MHz clock speed) and set mode to zero
// each loop takes about 3.8 uS so this limits the accuracy of the edge detection time to about +/- 2 us
// normal return value is uS since usStart
uint32_t waitHigh(void) {
  for (uint16_t timeout = 0xFFFF; timeout; timeout--) {
    if (digitalRead(PPM_IN)) return micros() - usStart;
  }
  mode = 0;
  return 0UL;
}

// similarly for low state
uint32_t waitLow(void) {
  for (uint16_t timeout = 0xFFFF; timeout; timeout--) {
    if (!digitalRead(PPM_IN)) return micros() - usStart;
  }
  mode = 0;
  return 0UL;
}

// function pointers used to swap to opposite polarity detection
typedef uint32_t (*wait)(void);
wait waitMain; // wait for main edge of interest (depends on polarity of signal)
wait waitOther; // wait for the other edge

// sample input once every usInterval microseconds.  count high samples and low samples.  exit after msDuration milliseconds.  return true if input changed state during routine
bool sample(uint32_t msDuration, uint16_t usInterval, uint16_t *highCount, uint16_t *lowCount) {
  *lowCount = *highCount = 0;
  uint32_t start = millis();
  uint32_t prev = micros();
  uint32_t next = prev + usInterval;
  uint32_t now;

  while ((millis() - start) < msDuration) {
    if (digitalRead(PPM_IN)) {
      (*highCount)++;
    } else {
      (*lowCount)++;
    }
    
    do {
      now = micros();
      
    } while ((now < next) && (now >= prev));
    
    prev = next;
    next += usInterval;
  }

  return *highCount && *lowCount;
}

void setup() {
  pinMode(PPM_IN, INPUT); 
  oled.init(0x3c);
  oled.clear();
  mode = 0;
}

void loop() {
  uint32_t us, smoothed, diff;
  uint16_t lowCount, highCount;
  char s[20];
  uint8_t image[2];
  uint8_t i, x;
  bool idle;
  
  switch (mode) {
    case 0: // sample input looking for high-going or low-going pulses
      pulse = 100UL << 4;
      if (sample(100, 100, &highCount, &lowCount)) {
        if (highCount > lowCount) {
          highPulses = false;
          waitMain = waitLow;
          waitOther = waitHigh;
        } else {
          highPulses = true;
          waitMain = waitHigh;
          waitOther = waitLow;
        }
        mode = 1;          
      } else {
        oled.cursorTo(0, 0);
        oled.printString    ("    PPM Tester    ");
        sprintf(s, "                  ");
        for (i = 1; i <= 7; i++) {
          oled.cursorTo(0, i);
          if (i == 2) { 
            oled.printString("   By ceptimus    ");
          } else if (i == 4) { 
            oled.printString("    No Signal     ");
          } else {
            oled.printString(s);
          }
        }
        for (i = 0; i < 12; i++) {
          reading[i] = 0UL;
        }
      }
      break;
    case 1: // wait for a main edge and capture usStart
      usStart = 0UL;
      if (!waitOther()) break;
      usStart = waitMain();
      if (usStart) {
        mode = 2;
      } else {
        mode = 0;
      }
      break;
    case 2: // wait for a sync interval > 4mS
      if (!waitOther()) break;
      us = waitMain();
      if (us) {
        usStart += us;
        if (us > 4000UL) {
          previous = 0UL;
          channel = 0;
          mode = 3;
        }
      }
      break;
    case 3: // measure (with smoothing) PPM channels till a sync interval is found, also capture smoothed pulse width
      us = waitOther();
      if (!us) break;
      pulse = pulse - (pulse >> 4) + us - previous;
      us = waitMain();
      if (us) {
        us -= previous;
        previous += us;
        if (us > 4000UL) {
          mode = 4;
        } else if (channel < 12) {
          smoothed = reading[channel] - (reading[channel] >> 4) + us;
          us <<= 4;
          diff = smoothed > us ? smoothed - us : us - smoothed;
          reading[channel++] = diff > 100UL ? us : smoothed;  // filter (smooth) the readings when they are fairly static, but jump straight to latest reading if they are changing
        }
      }
      break;
    case 4: // display results
      if (!channel || ((pulse >> 4) > 1000UL)) { // no channels or pulse too long - signal has disappeared or changed polarity
        pulse = 1600UL; // reinitialize pulse smoother to 100 us pulse width
        mode = 0; // start over
        break;
      }
      us = previous / 1000UL; // integer part of frame width in ms
      smoothed = (previous % 1000UL) / 100UL; // tenths of ms part of frame width
      sprintf(s, "%2ld.%ld ms  %5ld us ", us, smoothed, (pulse >> 4)); // display frame duration in milliseconds and pulse width in microseconds
      oled.cursorTo(0, 0);
      oled.printString(s);
      for (i = 0; i < 12; i++) { // display up to twelve channels using microsecond values
        if (i < channel) {
          sprintf(s, "%4ld  ", reading[i] >> 4);
        } else {
          sprintf(s, "      ");
        }
        oled.cursorTo((i % 3) * 42, 2 + i / 3);
        oled.printString(s);
      }
      // now draw the graph trace
      previous /= 128UL; // divide by this to map frame width to 128-pixel wide display
      i = 0; // channel
      idle = true;
      smoothed = 1000UL / previous; // draw first edge after 1ms lead in
      for (x = 0; x < 126; x++) { // only drawing the first 126 pixels as the last two are never used (or cleared) by text
        if (x == smoothed) { // draw transition
          image[0] = 0xFC;
          image[1] = 0xFF;
          if (idle) { // maybe draw the pulse more than one pixel across
            if ((pulse >> 4) / previous) {
              idle = false;
              smoothed += (pulse >> 4) / previous;
            } else { // pulse is too narrow to display so just display a one-pixel wide spike
              if (i < channel) {
                smoothed += (reading[i++] >> 4) / previous;
              } else {
                smoothed = 128;          
              }
            }
          } else { // drawing the trailing edge of a pulse
            idle = true;
            smoothed -= (pulse >> 4) / previous;
            if (i < channel) {
              smoothed += (reading[i++] >> 4) / previous;
            } else {
              smoothed = 128;
            }
          }
        } else { // draw high or low state
          if (idle) { // draw idle state
            image[0] = highPulses ? 0x00 : 0x04;
            image[1] = highPulses ? 0x80 : 0x00;
          } else { // draw active state
            image[0] = highPulses ? 0x04 : 0x00;
            image[1] = highPulses ? 0x00 : 0x80;
          }
        }
        oled.drawImage(image, x, 6, 1, 2);
      }
      mode = 1;
      break;
  }
}
